function [pl,w1,pl1]=add_noise(w,perr,r)

% Add uncertainty to categorical attributes
%
% INPUTS
%
% w: (n,p) matrix of attributes
% perr: error probability
% r: vector of length p; r(j) is the number of modalities of attribute j
%
% OUTPUT
%
% pl: (n,p,R) matrix of attribute value plausibilities; pl(i,j,h) is the
%      plausibility that attribute j takes value h for object i
% w1: (n,p) matrix of corrupted (noisy) labels
% pl1: same information as w1, coded in same format as pl
%
% Written by Thierry Denoeux, 11 Augut 2011.


[n,p]=size(w);
pl=zeros(n,p,max(r));
pl1=pl;


w1=w;
for i=1:n,
    for j=1:p,
        x=rand;
        if x<perr(i,j),
            w1(i,j)= unidrnd(r(j));
        end;
        if isnan(w1(i,j)),
            pl(i,j,1:r(j))=ones(1,r(j));
            pl1(i,j,1:r(j))=ones(1,r(j));
        else,
            pl(i,j,1:r(j))= perr(i,j)*ones(1,r(j))/r(j);
            pl(i,j,w1(i,j))=pl(i,j,w1(i,j))+1-perr(i,j);
            pl1(i,j,w1(i,j))=1;
        end;
    end;
end;

pl=squeeze(pl);
pl1=squeeze(pl1);
